/*==========================================================================
    Frictional motion (Burridge-Knopoff model)
    $B0lDjB.EY(BV$B$G0\F0$9$k>2$+$iDq93NO(Bf$B$r<u$1$F!"%P%M$G7R$,$l$?J*BN$,9T$&1?F0!#(B

    (u_i,v_i): $B0LCV$HB.EY(B i=1,2,...N
      du_i/dt=v_i
      dv_i/dt=-u_i+f(V-v_i)+k(u_i+1+u_i-1-2.0*u_i)

     f(z)=bz                   (0<=zb<1)
     f(z)=2-bz;                (1<=zb<1+$B&R(B)
     f(z)=c'/(1+2cz/(1.0-$B&R(B))  (1+$B&R(B<=z*b)
     f(-z)=f(z), f(z) is continuous at zb=1,1+$B&R(B. <==> c'=1-$B&R(B+2c(1+$B&R(B)/b

     $B"((B b$B"*!g(B limit was used by Langar and Carlson.
==========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>


/************** $BBg0hJQ?t$H4X?t@k8@(B ********************/

double          V,b,c,k,DU,sigma=0.1;
double          *u,*v;
int             N;
double          t;

void            initial(int);
void            boundary(void);

double f(double z){
  double w;
  if(fabs(z*b)<1.0){             w=b*fabs(z);}
  else if(fabs(z*b)<1.0+sigma){  w=2.0-b*fabs(z);}
  else{ w=(1.0-sigma+2.0*c*(1+sigma)/b)/(1.0+2.0*c*fabs(z)/(1.0-sigma));}
  if(z>0.0) return w; else return -w;
}


/*---------------- $B0J2<(B XWindow $BMQ(B --------------------*/
void            draw(int);
void            explain(int)
{  if(!fork()){ execlp("evince","evince","README_FrictionalMotion_1d.pdf",NULL);}  }


int main(int argc,char **argv)
{
   double    DT;

   /*--------------- initialize ---------------------*/

   int ar=0;
   ar++; if(argc>ar) N =atoi(argv[ar]);     else N =250;
   ar++; if(argc>ar) DT=atof(argv[ar]);     else DT=0.001;
   ar++; if(argc>ar) V =atof(argv[ar]);     else V =0.0015;
   ar++; if(argc>ar) b =atof(argv[ar]);     else b =1000.0;
   ar++; if(argc>ar) c =atof(argv[ar]);     else c =1.0;
   ar++; if(argc>ar) k =atof(argv[ar]);     else k =1.0;
   ar++; if(argc>ar) sigma=atof(argv[ar]);  else sigma=0.0;
   ar++; if(argc>ar) DU=atof(argv[ar]);     else DU=0.1;

   fprintf(stderr,"usage%% FBurridgeKnopff [N] [DT] [V] [b] [c] [k]\n"
          " element number: N=%i\n"
          " current parameter : V=%g b=%g c=%g k=%g\n"
	  " time meshsize: DT=%g\n"
	   ,N,V,b,c,k,DT); 


   u  = new double [N+2];      v  = new double [N+2];
   double   *u2,*v2;
   u2 = new double [N+2];      v2 = new double [N+2];   

   /*--------------- main roution -------------------*/

   initial(0);
   while(1){
       draw(0);

       double du;
       for(int i=1;i<N+1;i++){
           du=u[i-1]+u[i+1]-2.0*u[i];
           u2[i]=u[i]+DT*v[i];
           v2[i]=v[i]+DT*(-u[i]+f(V-v[i])+k*du);
       }
	 
       for(int i=1;i<N+1;i++){  u[i]=u2[i]; v[i]=v2[i];  }

       boundary();
       t+=DT;      
    }
}

/**********************  $B6-3&>r7o(B *************************/

void boundary(void)
{
    v[0]=v[1];
    v[N+1]=v[N];
    u[0]=u[1];
    u[N+1]=u[N];
}

/**********************  $B=i4|>r7o(B *************************/

#include <boost/random.hpp>
using namespace std;

void initial(int n)
{
    using namespace boost;
    // $B%a%k%;%s%L%D%$%9%?!<K!$r(B seed=4259 $B$G;H$&$3$H$r@k8@(B
    mt19937            gen(4259);
    //$B!V(B0.0$B!A(B1.0$B$N<B?t$N0lMMMp?t!W$r@8@.$5$;$k@_Dj(B
    uniform_real<>     dst(-1.0,1.0);
    variate_generator<mt19937, uniform_real<> > rand( gen, dst );


   for(int i=0; i<N+2; i++){ u[i]=DU*rand(); v[i]=0.0; }
   boundary();
   t=0.0; 
   draw(1);
}

/********************** $B=q$/(B *************************/
#include <XRS/Control.h>
#include <XRS/Simple.h>
#include <XRS/Plane.h>
using namespace XRS;

Control_Window  CW;
Simple_Window    W;
Plane_Window    PW;
double          ampu=0.4,ampv=1.0,ampW=100.0;
int             Wimax,Wjmax,Wjoffset,PWsize=400; 
int             Di=2,J0;
int             Dispstep=30000;
int             WinJ(double u,double a){ return J0-(int)(a*u);  }
double          PWinX(int i, double a){ return (double(i)/double(PWsize)-0.5)/a; }
int             PWinI(double x,double a){ return int(((x)*a+0.5)*PWsize) ;       }

void draw(int clear)
{
  static int Initial=1;    
  static int ts=0;
    
  if(Initial){
       /*---------------- setup window -------------------*/
       Wimax=N*Di; Wjmax=600; Wjoffset=100; J0=Wjmax-Wjoffset;
       CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
       CW.realtime=1;
   
       CW<<t<<"t"<<PARAMETER
	 << V << b << c << sigma << k << Dispstep <<ampu  << ampv << ampW 
	 <<"V"<<"b"<<"c"<<"sigma"<<"k"<< "slow"   <<"ampu"<<"ampv"<<"ampW";
   
       W.geometry(Wimax,Wjmax,1)<<initial
       <<"1d stick-slip system( A mouse click sets up the initial condition. )";

       initialize();
       Initial=0;
   }		 

   checkevent();

   static double x0=0.0;
   if(clear){ W.clear(); x0=0.0;  }
   
   if(ts<Dispstep){ ts++; return; } ts=0; 

   double j1=WinJ(-u[1]+V*t-x0,ampW);
   for(int i=2; i<N+1; i++){
       double j2=WinJ(-u[i]+V*t-x0,ampW);
       XDrawLine(W.disp(),W,W.bgc,Di*(i-1),j1,Di*i,j2);
       j1=j2;
   }
   if(WinJ(V*t-x0,ampW)<Wjoffset){ W.clear(); x0=V*t;}


   PW.geometry(PWsize,PWsize,1)<<"u-v";
   PW.draw_line(2,PWinI(0,ampv),0,PWinI(0,ampv),PWsize);	
   PW.draw_line(2,0,PWsize-PWinI(0,ampu),PWsize,PWsize-PWinI(0,ampu));

   int j0,k0,j,k;
   PW.clear(1);   
   double x=PWinX(0,ampv); j0=PWsize-PWinI(f(V-x),ampu);
   for(int i=1; i<PWsize; i++){
       x=PWinX(i,ampv); j=PWsize-PWinI(f(V-x),ampu);
       PW.draw_line(1,i-1,j0,i,j);
       j0=j;
   }
   PW.clear(0);
   j0=PWsize-PWinI(u[0],ampu); k0=PWinI(v[0],ampv);
   for(int i=1; i<N+1; i+=2){
      j=PWsize-PWinI(u[i],ampu); k=PWinI(v[i],ampv);
      PW.draw_line(0,k0,j0,k,j);
      j0=j; k0=k;
   }   
}




