/*==========================================================================
   extended Fisher-Kolmogorov(EFK) equation

     (∂t)U = (∂xx)U - γ(∂xxxx)U + U*(1-U*U)
===========================================================================*/

/*********** include **************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <XRS/Control.h>
#include <XRS/Plane.h>
#include <XRS/Simple.h>
using namespace XRS;

/*********** define ***************/

#define PI          M_PI
#define RAN         (drand48()-0.5)
#define integer(x)  ((int)(floor(x)+floor(2*((x)-floor(x)))))


/*********** global variable ************/

double                  t;
int                     N;
double                  *u;

double                  L;
double                  ga=0;
double                  DT,DX;
double                  disp=1,dispx=1,r_start,quit_time;
int                     i_max,j_max,init=0,DISPSTEP=10;

Display                 *d;
Plane_Window            win;
Simple_Window           ps_win;
int                     ps_wimax=200,ps_wjmax=200;
Control_Window          CW;
GC                      black,white;


/********** function & class prototype *****************/

void                    initial(int);
void                    draw(int);
void                    save(int);


/********** main ***********/

int main(int argc,char **argv)
{
     int ar=0;
     ar++; if(argc>ar) L =atof(argv[ar]); else L=200.0;
     ar++; if(argc>ar) DX=atof(argv[ar]); else DX=1.0;
     ar++; if(argc>ar) DT=atof(argv[ar]); else DT=0.001;
     fprintf(stderr,"Usage: EFKeq L DX DT\n");
     fprintf(stderr," L=%g, DX=%g, DT=%g\n",L,DX,DT);     

     N=integer(L/DX);
     i_max=N;   
     j_max=100;
     r_start=5;

     u =new double [N+1];
     double *u2=new double [N+1];

/*----- initialize ----------*/

     CW<< initial << t <<PARAMETER<< disp << dispx << ga
       <<"initial"<<"t"           <<"disp"<<"dispx"<<"ga";

     win.geometry(i_max,j_max)
        <<COLOR()<<"EFK equation";
     ps_win.geometry(ps_wimax,ps_wjmax)<<"phase space";
                        
     initialize();  
     d=win.disp(); 
     white=graphic_context("white");
     black=graphic_context("black");

     initial(0);
     draw(1);

     double du2,du4,tx2,tx4;
     tx2=DT/(DX*DX);
     tx4=DT/(DX*DX*DX*DX);

/*----- main routine --------*/

     while(1){
        checkevent();

        for(int ts=0; ts<DISPSTEP; ts++){
            for(int j=2;j<N-1;j++){
                du2=u[j+1]+u[j-1]-2.0*u[j];
		du4=u[j+2]+u[j-2]-4.0*(u[j+1]+u[j-1])+6.0*u[j];
                u2[j]=u[j]+tx2*du2-ga*tx4*du4;
		u2[j]+=DT*u[j]*(1-u[j]*u[j]);
            }

            for(int j=2;j<N-1;j++)  u[j]=u2[j];

            u[0]=u[3]; u[N]=u[N-3];
            u[1]=u[2]; u[N-1]=u[N-2];
	    
           t += DT;
           draw(0);
	   if(init) init=0;
        }
    }

    delete [] u;    delete [] u2;
}


/*************** initialize function ************/

void initial(int nn)
{
    for(int j=0; j<N+1; j++)
       if(DX*j<r_start) u[j]=1.0; else u[j] = 0.0;
 
    init=1;  t=0;
}


void draw(int initial)
{
#ifndef XRSBG
    static int *point=new int [N];
    static XPoint *phase=new XPoint [N];
    double Amp=0.4*disp*(double)j_max;
    double Ampy=0.4*disp*(double)ps_wjmax;
    double Ampx=dispx*(double)ps_wimax/(2.0*DX);

    if(initial){ win.clear(); ps_win.clear();  }

    XDrawLines(d,ps_win,white,phase+1,N-1,CoordModeOrigin);

    for(int m=1; m<N; m++){
       int j=(int)(0.5*(double)j_max-Amp*u[m]);
       if(point[m]!=j){
          win.erase_point(0,m,point[m]); 
          win.draw_point(0,m,j);
          point[m]=j;
       }
       phase[m].y=(int)(0.5*(double)ps_wjmax-Ampy*u[m]);
       phase[m].x=(int)(0.5*((double)ps_wimax+Ampx*(u[m+1]-u[m-1])));      
    }

    XDrawLines(d,ps_win,black,phase+1,N-1,CoordModeOrigin);
#endif
}

