/*==========================================================================
  FitzHugh-Nagumo equation with local diffution coupling in a one-dimensional space.

     ∂t u = f(u) - v  + Du ∂xx u
     ∂t v = ε(u-γv) + Dv ∂xx v
     f(u)=u(1-u)(1+u)

     system: 0<x<L, periodic boundary 
==========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Graphic.h>
#include <XRS/Plane.h>
using namespace XRS;

/************** 大域変数と関数宣言 ********************/

double          L,DT,DX,Du,Dv,ep,ga;  // パラメータ
double          u0=2.5,r0=0.6;        // 初期値の高さと範囲
double          *u,*v;                // 配列
double          t;                    // 時刻
int             N;                    // 配列のサイズ

double          f(double u){ return u*(1.0-u)*(1.0+u); }
void            initial(int);

/*------------ 描画用の変数と関数 ---------------*/
Control_Window  CW;
Plane_Window    W,PW;
int             Wimax,Wjmax,PWsize=200,Dispstep=500;
void            draw();                // 空間プロット
void            PW_draw(int);          // 相図プロット


int main(int argc,char **argv)
{
   /*--------------- initialize ---------------------*/
   int ar=0;
   ar++; if(argc>ar) L=atof(argv[ar]);    else L=100;
   ar++; if(argc>ar) DX=atof(argv[ar]);   else DX=0.25;
   ar++; if(argc>ar) DT=atof(argv[ar]);   else DT=0.00005;
   ar++; if(argc>ar) Du=atof(argv[ar]);   else Du=0.4;
   ar++; if(argc>ar) Dv=atof(argv[ar]);   else Dv=250.0;
   ar++; if(argc>ar) ep=atof(argv[ar]);   else ep=3.0;
   ar++; if(argc>ar) ga=atof(argv[ar]);   else ga=2.0/3.0;   

   fprintf(stderr,
     "usage%% FigtzhughNagmo_1d [L] [DX] [DT] [Du] [Dv] [ep] [ga]\n" 
     "system size: L=%g   meshsize: DX=%g  DT=%g\n"
     " parameters: Du=%g  Dv=%g  ep=%g  ga=%g\n"
     ,L,DX,DT,Du,Dv,ep,ga); 

   double   *u2,*v2,du,dv;
   N=(int)(L/DX); 
   u =new double [N];  v =new double [N];   
   u2=new double [N];  v2=new double [N];   

   /*--------------- main roution -------------------*/
   initial(0);

   while(1){
      draw();
      double txxu=Du*DT/(DX*DX);
      double txxv=Dv*DT/(DX*DX);
      double te=DT*ep;

      for(int ts=0; ts<Dispstep;ts++){
         for(int n=1;n<N-1;n++){ du=u[n-1]+u[n+1]-2.0*u[n];  u2[n]=u[n]+txxu*du+DT*(f(u[n])-v[n]); }
         for(int n=1;n<N-1;n++){ dv=v[n-1]+v[n+1]-2.0*v[n];  v2[n]=v[n]+txxv*dv+te*(u[n]-ga*v[n]); }
         for(int n=1;n<N-1;n++){ v[n]=v2[n]; u[n]=u2[n];}
             
         v[0]=v[N-2]; v[N-1]=v[1];
         u[0]=u[N-2]; u[N-1]=u[1];
      }

      t+=DT*Dispstep;      
    }
}


/**********************  初期条件 *************************/

double Wy(int j){ return double (0.5-(double)(j)/Wjmax)/0.3;}
int Wj(double y){ return (int)(Wjmax*(-0.3*y+0.5));         }		  

void initial(int mouse)
{
   if(mouse){ r0=double(W.ev.xbutton.x)/Wimax;  u0=Wy(W.ev.xbutton.y);  }

   for(int n=0; n<N; n++){ u[n]=v[n]=0.0; }
   int n0=r0*N;
   if(n0>=N/2){ for(int n=N-n0; n<n0; n++)  u[n]=u0;}
   else       { for(int n=n0; n<N-n0; n++)  u[n]=u0;}    
   
   v[0]=v[N-2]; v[N-1]=v[1];
   u[0]=u[N-2]; u[N-1]=u[1];

   t=0; 
}


/********************** 描画 *************************/

void draw(void)
{
    int j,j0;
    static int Initial=1;

    if(Initial){ //最初に1回だけやること
        Wimax=N; Wjmax=200;    
        CW<<t<<"t"<<PARAMETER<<ep<<"ep"<<ga<<"ga"<<Du<<"Du"<<Dv<<"Dv"; 
        W.geometry(Wimax,Wjmax,1)<<initial<<"FitzHugh-Nagmo equation";
        PW.geometry(PWsize,PWsize,1)<<"u-v";	   
        initialize();
    }

    checkevent();  //マウスクリックのチェック

    if(Initial || CW.chev){ //最初とマウスクリック時にやること
        W.clear();
        W.draw_line(3,Wimax/2,0,Wimax/2,Wjmax); 	
        W.draw_line(3,0,Wj(0),Wimax,Wj(0));
        W.draw_line(2,0,Wj(1),Wimax,Wj(1));
        W.draw_line(2,0,Wj(-1),Wimax,Wj(-1));
        PW_draw(1);
        W.chev=0;
        Initial=0;
    }

    // 以下は毎回やる。
    PW_draw(0);		 

    W.clear(0);
    j0=Wj(u[0]); for(int n=0; n<N; n++){ j=Wj(u[n]);  W.draw_line(0,n-1,j0,n,j);  j0=j;  }   

    W.clear(1);  
    j0=Wj(v[0]); for(int n=0; n<N; n++){ j=Wj(v[n]);  W.draw_line(1,n-1,j0,n,j);  j0=j;  }   

    XFlush(W.disp());
}



double PWu(int    j){ return ((double)j/PWsize-0.5)/0.3; }   // (u,v) in the phase space 
int    PWj(double u){ return (int)(PWsize*(0.3*u+0.5));  }   //   ↑
double PWv(int    k){ return (0.5-(double)k/PWsize)/0.3; }   //   ↓
int    PWk(double v){ return (int)(PWsize*(-0.3*v+0.5)); }   // (j,k) in PW window

void PW_draw(int init)
{
    int j,k,j0,k0;

    if(init){ 
      PW.clear();	
      PW.draw_line(1,PWj(0),0,PWj(0),PWsize);	
      PW.draw_line(1,0,PWk(0),PWsize,PWk(0));

      double U=PWu(0); k0=PWj(f(U));
      for(j=1; j<PWsize; j++){ U=PWu(j); k=PWk(f(U)); PW.draw_line(2,j-1,k0,j,k); k0=k;}
      double V=PWk(0); j0=PWj(ga*V);
      for(k=1; k<PWsize; k++){ V=PWv(k); j=PWj(ga*V); PW.draw_line(3,j0,k-1,j,k); j0=j;}
    }

    PW.clear(0);
    j0=PWj(u[0]); k0=PWk(v[0]);
    for(int n=1; n<N; n+=2){  j=PWj(u[n]); k=PWk(v[n]);  PW.draw_line(0,j0,k0,j,k);  j0=j; k0=k;  }   
}    


