/*---------------------------------------------------------------------------
  Phase Field Model of crystal growth.

   τε^2 ∂t P = ε^2 △P+P*(1-P)*(P-0.5+m(T,θ))
   ∂t T = △T+K∂t P
   
   m(T,θ) = α f(θ) atan(β(1-T))/π
   f(θ)=1-σcos^2(3θ)
   (cosθ,sinθ)=∇P/|∇P|

   initial condition
   T(x,y)=T0

---------------------------------------------------------------------------*/

/*********** include **************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Graphic.h>
using namespace XRS;

//#include <import.h>

#define PI          M_PI
#define integer(x)  ((int)(floor(x)+floor(2*((x)-floor(x)))))
#define RATIO       0.866025403784439   // =sqrt(3)/2          
#define XIJ(i,j)    (int)((i)+0.5*((j)-N)+0.5)
#define YJ(j)       (int)(RATIO*(j)+0.5)

/************** 大域変数と関数宣言 ********************/

double          t;
double          **P,**T;
double          DT,DX;
double          L,alpha,beta,tau,K,e,sigma;
double          r_start,T0;
int             N,Wimax,Wjmax,Dispstep=10;

void            initial(int);
inline double   M(double x,double theta){
     double co=cos(3.0*theta);
     return alpha*(1.0-sigma*co*co)*atan(beta*(1-x))/PI;
}

void            boundary(void);
void            draw(int);
double          reverse(double x){ return 0.9*(1-x); }
double          linear(double x){ return 0.8*x; }


/*---------------- 以下 XWindow 用 --------------------*/

Control_Window  CW;
Graphic_Window  PW,TW;
int             PWsize,Initial=0;
void            explain(int)
{  if(!fork()){ execlp("evince","evince","README_SnowCrystal.pdf",NULL);}  }


/********** main ***********/

int main(int argc,char **argv)
{
   int ar=0;
   ar++; if(argc>ar)  L    =atof(argv[ar]);  else L=10;
   ar++; if(argc>ar)  DX   =atof(argv[ar]);  else DX=0.05;
   ar++; if(argc>ar)  DT   =atof(argv[ar]);  else DT=0.0005;
   ar++; if(argc>ar)  K    =atof(argv[ar]);  else K=1.50;
   ar++; if(argc>ar)  T0   =atof(argv[ar]);  else T0=0.0;
   ar++; if(argc>ar)  sigma=atof(argv[ar]);  else sigma=0.2;
   ar++; if(argc>ar)  alpha=atof(argv[ar]);  else alpha=0.9;
   ar++; if(argc>ar)  beta =atof(argv[ar]);  else beta=10.0;   
   ar++; if(argc>ar)  tau  =atof(argv[ar]);  else tau=3.0;
   ar++; if(argc>ar)  e    =atof(argv[ar]); else e=0.01;

   r_start=0.05*L;

   fprintf(stderr,
     "usage%% SnowCrystal [L] [DX] [DT] [K] [T0] [sigma] [alpha] [beta] [tau] [e]\n"
     " current systemsize: L=%g\n"
     " space and time meshsize: DX=%g DT=%g\n"
     " current parameter : K=%g T0=%g sigma=%g\n"
     "                   : alpha=%g beta=%g tau=%g e=%g\n"
     ,L,DX,DT,K,T0,sigma,alpha,beta,tau,e); 

   N=(int)(L/DX);  
   Wimax=Wjmax=2*N;  
   array(P,Wimax+1,Wjmax+1); 
   array(T,Wimax+1,Wjmax+1);

   double   **T2,**P2,**P3;
   array(P2,Wimax+1,Wjmax+1); array(P3,Wimax+1,Wjmax+1);
   array(T2,Wimax+1,Wjmax+1);

/*----- initialize ----------*/
   
   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   CW.realtime=1;

   CW<< t << initial<<"t"<<"initial"<<PARAMETER
     << alpha << beta << K << tau << e << sigma << T0
     <<"alpha"<<"beta"<<"K"<<"tau"<<"e"<<"sigma"<<"T0";
   
   PW.geometry(Wimax+1,YJ(Wjmax+1),1)<<"P(order parameter)"<<reverse<<MONO(0.52,20);
   TW.geometry(Wimax+1,YJ(Wjmax+1),1)<<"T(temperature)"<<linear<<MONO(0.04,20);

   initialize();  

   int i,j,ts;

/*----- main routine --------*/

   double   nl,dP,dT,tee,txx,txxp,theta,Px,Py;
   int      n,m;
   
   initial(0);
   draw(Initial);
   
   while(1){
      checkevent();

      tee=DT/(e*e*tau);
      txx=2.0*DT/(3.0*DX*DX);
      txxp=txx/tau;

/*----- P local map -----*/

      for(ts=0; ts<Dispstep;ts++){
        for(i=1;i<Wimax;i++)
          for(j=1;j<Wjmax;j++){
	     n=i-N; m=j-N;
	     if(m+n>-N && m+n<N){
	       Px=0.5*(P[i+1][j-1]-P[i-1][j+1]+P[i][j+1]-P[i][j-1])
 		  +P[i+1][j]-P[i-1][j];
	       Py=RATIO*(P[i-1][j+1]-P[i+1][j-1]+P[i][j+1]-P[i][j-1]);
	       theta=atan2(Px,Py);
               nl=P[i][j]*(1-P[i][j])
	         *(P[i][j]-0.5+M(T[i][j],theta));
               P2[i][j]=P[i][j]+tee*nl;
	     }
          }

/*----- P & T diffusion term -----*/

        for(i=1;i<Wimax;i++)
          for(j=1;j<Wjmax;j++){
	     n=i-N; m=j-N;
	     if(m+n>-N && m+n<N){
               dP=P[i+1][j-1]+P[i-1][j+1]+P[i-1][j]+P[i+1][j]
		 +P[i][j-1]+P[i][j+1]-6.0*P[i][j];
               P3[i][j]=P2[i][j]+txxp*dP;
               dT=T[i+1][j-1]+T[i-1][j+1]+T[i-1][j]+T[i+1][j]
		 +T[i][j-1]+T[i][j+1]-6.0*T[i][j];
               T2[i][j]=T[i][j]+txx*dT;
	     }
          }

/*----- T local map -----*/
        for(i=1;i<Wimax;i++)
          for(j=1;j<Wjmax;j++){
	     n=i-N; m=j-N;
	     if(m+n>-N && m+n<N){
               T[i][j]=T2[i][j]+K*(P3[i][j]-P[i][j]);
               if( T[i][j] < 0 ) T[i][j]=0.0;
               P[i][j]=P3[i][j];
	     }
          }
	
        boundary();
      }
      
      t+=DT*Dispstep;
      draw(Initial);
      if(Initial) Initial=0;
   }
}


/**********************  初期条件 *************************/

void initial(int)
{
    int n,m;
    
    for(int j=0; j<Wjmax+1; j++)
        for(int i=0; i<Wimax+1; i++){ P[i][j] = 0; T[i][j] = T0; }

    for(int j=1; j<Wjmax; j++)
        for(int i=1; i<Wimax; i++){
	    n=XIJ(i,j)-XIJ(N,N); m=YJ(j)-YJ(N);    
            if(n*n+m*m<(r_start/DX*r_start/DX)){
                T[i][j] = 1.0; P[i][j] = 1.0;
            }
	}

    boundary();
    t=0; Initial=0;
}


/**********************  境界条件 *************************/

void boundary(void)
{
    for(int j=0;j<N;j++)    { T[Wimax][j]=T[Wimax-2][j+1]; P[Wimax][j]=0; }
    for(int j=N;j<Wjmax;j++){ T[0][j]=T[2][j-1];           P[0][j]=0;     }
    for(int i=1;i<N;i++)    { T[i][Wjmax]=T[i+1][Wjmax-2]; P[i][Wjmax]=0; }
    for(int i=N;i<Wimax;i++){ T[i][0]=T[i-1][2];           P[i][0]=0;     }
    for(int i=1;i<N;i++)    { T[i][N-i]=T[i+1][N-i+1];     P[i][N-i]=0;   }
    for(int i=N;i<Wimax;i++){ T[i][N-i+Wjmax]=T[i-1][N-i+Wjmax-1];
                              P[i][N-i+Wjmax]=0;   }
    T[0][N]    =0.5*(T[1][N+1]+T[2][N-1]);              P[0][N]=0;
    T[Wimax][N]=0.5*(T[Wimax-2][N+1]+T[Wimax-1][N-1]);  P[Wimax][N]=0;
    T[N][0]    =0.5*(T[N+1][1]+T[N-1][2]);              P[N][0]=0;    
    T[N][Wjmax]=0.5*(T[N+1][Wjmax-2]+T[N-1][Wjmax-1]);  P[N][Wjmax]=0;
    T[0][Wjmax]=0.5*(T[1][Wjmax-2]+T[2][Wjmax-1]);      P[0][Wjmax]=0;
    T[Wimax][0]=0.5*(T[Wimax-2][1]+T[Wimax-1][2]);      P[Wimax][0]=0;    
}


void  draw(int)
{
    if(Initial){ PW.clear(); TW.clear();}

    int n,m;
    
    for(int i=0;i<Wimax+1;i++){
       for(int j=0;j<Wjmax+1;j++){
	  n=i-N; m=j-N;
          if(m+n>=-N && m+n<=N){
	     PW.draw_point(P[i][j],XIJ(i,j),YJ(j));
	     TW.draw_point(T[i][j],XIJ(i,j),YJ(j));
	  }
       }
    }

    /*
    XFlush(PW.disp());
    static int count=0;
    char fileP[256],fileT[256];
    printf("t=%g\n",t);

    sprintf(fileP,"P.%04d.jpg",count); 
    import(PW,fileP);
    sprintf(fileT,"T.%04d.jpg",count); 
    import(TW,fileT);
    count++;
    */
}  






    


